/* ----------------------------------------------------------
                           codedcolor.c

     Arithmogriph Zahlenraetselgenerator nach folgendem
     Schema:

     a + b = c             400 + 500 = 900
     -   +   -              -     +     -
     d - e = f             300 - 200 = 100
     _________             _______________
     g + h = i             100 + 700 = 800


     Fuer positive Zahlen in den Ergebnissen gilt:

     a > d, d > e

     MCU   :  STM32F103 / BluePill
     Takt  :  72 MHz

     08.12.2023  R. Seelig
   ---------------------------------------------------------- */


#include <stdint.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

#include <libopencm3.h>

#include "sysf103_init.h"
#include "tftdisplay.h"
#include "my_printf_s.h"
#include "gfx_pictures.h"
#include "spiro.h"
#include "keypad.h"

#include "./src/coded_color_screen.c"

#define printf       my_printf


#define SAVEADR1     0x08007c00               // Speicherpageadresse des Flashs im Controller,
                                              // ab der insgesamt 1 KByte (eine Page) gespeichert werden kann
                                              
#define eep_write16(adr, value)        { flash_program_word(SAVEADR1 + (adr << 2), value); delay(1); }
#define eep_read16(adr)                *((uint32_t*)(SAVEADR1 + (adr << 2)))  


// Schriftstilzuordnung
enum { fnt8x8, fnt12x16, fnt5x7 };

// Displaypositionen fuer die farbigen Kloetzchen oder einer Ziffer
#if (_yres > 128)
  #define sym_ofsx       22
#else
  #define sym_ofsx       6
#endif
#define sym_ofsy       6

/* -------------------------------------------------------------
                           globale Variable
   ------------------------------------------------------------- */

//  Farbnummern nach der Widerstandsfarbreihe, Elektroniker halt :-)
uint16_t cols565[10] = {
//  sw      br      rt      or      ge      gn      bl      vl      gr      ws
    0x4208, 0xc402, 0xf945, 0xfd86, 0xffe0, 0x0680, 0x421f, 0xf95f, 0x8410, 0xffff

//  0x2948, 0x81e0, 0xf945, 0xfbc5, 0xffe0, 0x0400, 0x295f, 0xf95f, 0x8410, 0xffff
};


// Ausgabeposition der Zahlen als in Koordinaten
uint8_t outpos_nums[18] =
{
  sym_ofsx+0, sym_ofsy+0,  sym_ofsx+38, sym_ofsy+0,  sym_ofsx+76,sym_ofsy+0,
  sym_ofsx+0, sym_ofsy+20, sym_ofsx+38, sym_ofsy+20, sym_ofsx+76,sym_ofsy+20,
  sym_ofsx+0, sym_ofsy+40, sym_ofsx+38, sym_ofsy+40, sym_ofsx+76,sym_ofsy+40
};

// Ausgabeposition der Termzeichen ( + - = )
// einzelner Frame besteht aus 3 Bytes: x,y,Zeichen
uint8_t outpos_ops[36] =
{
  sym_ofsx+30, sym_ofsy+1,  '+', sym_ofsx+68, sym_ofsy+1,  '=',
  sym_ofsx+10, sym_ofsy+11, '-', sym_ofsx+48, sym_ofsy+10, '+', sym_ofsx+97, sym_ofsy+10,'-',
  sym_ofsx+30, sym_ofsy+21, '-', sym_ofsx+68, sym_ofsy+21, '=',
  sym_ofsx+10, sym_ofsy+31, '=', sym_ofsx+48, sym_ofsy+31, '=', sym_ofsx+97, sym_ofsy+31, '=',
  sym_ofsx+30, sym_ofsy+41, '+', sym_ofsx+68, sym_ofsy+41, '='
};

uint8_t rectpos;                // speichert die Position des Auswahlquadrats beim Anwaehlen
                                // eines Feldes

uint16_t anum[9];               // nimmt die 9 Integerzahlen der Rechenoperation auf
                                // und somit das Raetsel

uint8_t  colzif[10];            // nimmt die Farbzuweisung einer Farbe zu einer Ziffer auf
                                // colzif[0] nimmt Ziffer fuer schwarz auf, [1] fuer rot etc.

uint8_t usersel[10];            // Nimmt die Ziffern auf, die der User den Farben zuordnet
                                // waehrend des Raetsels wird Bit7 als Anzeigeflag genutzt:
                                // Flag nicht gesetzt: Farbe wird angezeigt
                                // Flag gesetzt:       durch User angegebene Ziffer wird angezeigt

                                // Bit 6 gibt Auskunft, ob eine Auswahl vorgenommen wurde
                                // Flag nicht gesetzt: Farbe wurde nicht zugeordnet
                                // Flag gesetzt      : Farbe wurde zugeordnet

uint16_t bkscrcolor= 0xad55;    // Hintergrundfarbe des Raetsels (0xad55 = hellgrau)
uint16_t rectcol;               // Farbe des Auswahlquadrats

uint16_t ziffcheck;

// Variable des Schieberegisters fuer Zufallszahlen
uint16_t start_state= 1;
uint16_t lfsr= 1;
char     first = 0;

uint16_t zstw;                  // Beinhaltet die Raetselnummer (0..9999)
uint8_t  newraetsel;            // Flag, ob ein neues Raetsel gestartet wird


/* --------------------------------------------------
   flash_init

   "unlocked" den Flash-Speicher und loescht den
   Speicherbereich der in SAVEADR1 angegeben ist
   -------------------------------------------------- */
void flash_init(void)
{
  flash_unlock();
  flash_clear_pgerr_flag();
  flash_clear_eop_flag();
  flash_clear_wrprterr_flag();
  flash_clear_pgerr_flag();

  flash_erase_page(SAVEADR1);            // 1 KByte aus Page loeschen
  delay(40);
}

/* -------------------------------------------------------
                         button_draw

     zeichnet einen beschrifteten Schaltbutton an den
     Koordinaten x,y mit dem Text in s und dem Schrift-
     stil fnt
   ------------------------------------------------------- */
void button_draw(uint16_t x, uint16_t y, char *s, uint8_t fnt, uint8_t active)
{
  uint8_t  len;
  uint16_t bktmp, txtmp;
  uint16_t fntx, fnty, oldfnt;
  uint16_t dk_col, lg_col;

  oldfnt= fontnr;
  setfont(fnt);
  if (fontsizex== 5) fntx= 6; else fntx= fontsizex;
  fnty= fontsizey;

  bktmp= bkcolor;
  txtmp= textcolor;
  len= strlen(s);
  dk_col= rgbfromvalue(0x20, 0x20, 0x20);
  lg_col= rgbfromvalue(0xff, 0xff, 0xff);

  fillrect(x,y, x+8+((len+1)*fntx), y+8+fnty, rgbfromvalue(0x70, 0x70, 0x70));
  if (!active)
  {
    line(x,y, x+8+((len+1)*fntx),y, lg_col);
    line(x,y, x,y+8+fnty, lg_col);

    line(x+1, y+8+fnty, x+8+((len+1)*fntx),y+8+fnty, dk_col);
    line(x+8+((len+1)*fntx),y+8+fnty, x+8+((len+1)*fntx),y+1, dk_col);

    textcolor= rgbfromvalue(0x00, 0x00, 0x00);
    outtextxy(x+9, y+6, 0, s);
  }
  else
  {
    line(x,y, x+8+((len+1)*fntx),y, dk_col);
    line(x,y, x,y+8+fnty, dk_col);

    line(x+1, y+8+fnty, x+8+((len+1)*fntx),y+8+fnty, lg_col);
    line(x+8+((len+1)*fntx),y+8+fnty, x+8+((len+1)*fntx),y+1, lg_col);

    textcolor= rgbfromvalue(0xcc, 0xcc, 0xcc);
    outtextxy(x+8, y+5, 0, s);
  }

  setfont(oldfnt);
  textcolor= txtmp;
  bkcolor= bktmp;
}

/* --------------------------------------------------------
                          my_putchar

   wird von my-printf / printf aufgerufen und hier muss
   eine Zeichenausgabefunktion angegeben sein, auf das
   printf dann schreibt !
   -------------------------------------------------------- */
void my_putchar(char ch)
{
  lcd_putchar(ch);
}

/* -------------------------------------------------------
                           swap16

              tauscht 2 16-Bit Integerwerte
   ------------------------------------------------------- */
void swap16(uint16_t *a, uint16_t *b)
{
  uint16_t tmp;

  tmp= *a;
  *a= *b;
  *b= tmp;
}

/* -------------------------------------------------------
                           swap8

              tauscht 2 8-Bit Integerwerte
   ------------------------------------------------------- */
void swap8(uint8_t *a, uint8_t *b)
{
  uint8_t tmp;

  tmp= *a;
  *a= *b;
  *b= tmp;
}


/* --------------------------------------------------------
                         random16

    generiert eine 16-Bit grosse Pseudozufallszahl die
    mittels mitgekoppeltem Schieberegister erreicht wird

    Uebergabe:
      startwert  : Ausgangspunkt im Zahlenbereich

    rndm_mask ist die XOR-Maske beim Mitkoppeln des
    Schieberegisters

          XOR-Maske nach     Anzahl unterschiedlichen
          dem Schieben       Zahlen im Ergebnis
          -------------------------------------------
              0xb400               65535
              0x07c1                2047
              0x0be0                4095
  -------------------------------------------------------- */
uint16_t random16(uint16_t startwert)
{

  uint16_t lsb;

  if (first== 0)
  {
    first= 1;
    start_state= startwert;
    lfsr= start_state;
  }

  lsb = lfsr & 1;                        // niederwertigstes Bit des Schieberegisters
  lfsr = lfsr >> 1;                      // das Schieberegister, eine Stelle nach rechts
  if (lsb) { lfsr ^= 0xb400; }           // wenn LSB gesetzt, XOR-Togglemaske auf SR anwenden

  return lfsr;
}

/* -------------------------------------------------------
                        rndm_permut10

     zufaellige Permutation von 10 Ziffern nach
     Fisher-Yates

     Hier: vielen Dank an Yalu aus www.mikrocontroller.net

     Uebergabe:
        *buf : Zeiger auf 10 Byte grosses Array in dem
               eine Ziffer von 0..9 jeweils nur ein
               einziges mal vorkommt
   ------------------------------------------------------- */
void rndm_permut10(uint8_t *buf, uint8_t startw)
{
  int8_t i,z;

  for (i= 0; i< 10; i++)
  {
    *(buf+i) = i;
  }
  for (i= 9; i> -1; i--)
  {
//    z= (random16(startw) % 10);
    z= (random16(startw) % (i+1));
    swap8(&(*(buf+i)), &(*(buf+z)));
  }
}

/* -------------------------------------------------------
                      generate_termnums

     generiert die Zahlen fuer den Term des Arithmogriph
     nach dem Schema:

     a + b = c
     -   +   -
     d - e = f
     _________
     g + h = i

     Fuer positive Zahlen in den Ergebnissen gilt:
     a > d, d > e

     h soll 3-stellig bleiben, damit Ergebnis auf einem
     kleinen Display darstellbar bleibt.
     Jede Zahl muss mindestens 2-stellig sein (damit es
     nicht zu einfach wird)

     Uebergabe:
        *buf : Zeiger auf ein 9 Elemente grosses
               Zahlenarray, dass die Zahlen entsprechend
               dem oben gezeigten Term aufnimmt.
               Element 0 entspricht "a" im obigen
               Arithmogriph
        zstw : Startwert fuer den Zufallszahlengenerator


   ------------------------------------------------------- */
void generate_termnums(uint16_t *buf, uint16_t zstw)
{
  do
  {
    buf[0]= (random16(zstw) % 988) + 11;             // a
    buf[1]= (random16(zstw) % 988) + 11;             // b
    buf[3]= (random16(zstw) % 988) + 11;             // d
    buf[4]= (random16(zstw) % 988) + 11;             // e

    // buf[0] muss aus den obigen Bedingungen die hoechste Zahl von a, d und e sein

    if (buf[3] > buf[0]) swap16(buf, buf+3);         // if d>a: a <==> d
    if (buf[4] > buf[0]) swap16(buf, buf+4);         // if e>a: a <==> e
    if (buf[4] > buf[3]) swap16(buf+4, buf+3);       // if e>d: e <==> d

    // das Raetsel berechnen
    buf[2]= buf[0] + buf[1];                         // c= a + b
    buf[5]= buf[3] - buf[4];                         // f= d - e
    buf[6]= buf[0] - buf[3];                         // g= a - d
    buf[7]= buf[1] + buf[4];                         // h= b + e
    buf[8]= buf[6] + buf[7];                         // i= g + h

  } while (*(buf+7) > 999);                          // wiederholen bis h < 1000 ist
}



/* -------------------------------------------------------
                        anz_ziffs

     zaehlt die Anzahl der Ziffern eines Integers und
     gibt die Anzahl als Argument zurueck
   ------------------------------------------------------- */
uint8_t anz_ziffs(uint16_t num)
{
  if (num> 9999) return 5;
  if (num> 999) return 4;
  if (num> 99) return 3;
  if (num> 9) return 2;
  return 1;
}

/* -------------------------------------------------------
                       my_uitoa

     konvertiert einen 16-bit Integerwert in einen
     String
   ------------------------------------------------------- */
void my_uitoa(uint16_t zahl, char *str)
{
  uint8_t  zif;
  uint8_t  i,l;

  l= anz_ziffs(zahl);
  str[l+1]= 0;                   // String Terminator
  for(i= l; i>=1; i--)
  {
    zif= zahl % 10;
    str[i-1]=zif +'0';
    zahl /= 10;
  }
}

/* -------------------------------------------------------
                      ziffanz_check
     testet, ob alle Ziffern 0..9 im Zahlenraetsel
     vorhanden sind wenn dem so ist, wird 0x03ff zurueck
     gegeben (vermeidet somit, dass eine Ziffer im
     Raetsel NICHT vorhanden ist)
   ------------------------------------------------------- */
uint16_t ziffanz_check(void)
{
  int      i,a;
  char     hstr[8];
  uint16_t ziff_check;
  char     *ptr;

  ziff_check= 0;
  for (a= 0; a< 9; a++)
  {
    my_uitoa(anum[a],hstr);
    for (i= 0; i< 10; i++)
    {
       ptr = strchr(hstr, (i+'0'));
       if (ptr) ziff_check |= 1 << i;
    }
  }

  return ziff_check;
}

/* -------------------------------------------------------
                        usersel_free

     testet, ob eine gewaehlte Zuordnung noch verfuegbar
     ist

     Rueckgabe:
       0 : nicht verfuegbar
       1 : verfuegbar
   ------------------------------------------------------- */
uint8_t usersel_free(uint8_t own, uint8_t value, uint8_t *sel)
{
  uint8_t i;

  if (value== 0x40) return 1;            // unmarkierte Plaetze sind immer frei
  for (i= 0; i< 10; i++)
  {
    if (i != own)                       // eigenen Wert nicht vergleichen
    {
      if (sel[i]== value) return 0;
    }
  }
  return 1;
}

/* -------------------------------------------------------
                        zif_cmp

     vergleicht die Permutation des Raetsels mit den
     gemachten Userzuordnungen (und testet so, ob das
     Raetsel geloest ist)

     Rueckgabe:
       0 : Raetsel nicht geloest
       1 : Raetsel geloest
   ------------------------------------------------------- */
uint8_t zif_cmp(void)
{
  uint8_t i;

  for (i= 0; i< 10; i++)
  {
    if (colzif[i] != (usersel[i] & 0x4f)) return 0;              // 0x4f  laesst das Unmarkiertflag stehen
  }                                                            // somit gewaehrleistet, dass alle Ziffern
  return 1;                                                    // zugeordnet sind
}

/* -------------------------------------------------------
                           zif_print

      zeigt einzelne Ziffer an
   ------------------------------------------------------- */
void zif_print(uint8_t x, uint8_t y, uint8_t zif)
{
  textcolor= rgbfromega(0);
  fillrect(x, y, x+9, y+9, bkscrcolor);
  putcharxy(x+1, y+2, zif+'0');

}

/* -------------------------------------------------------
                          symzif_print

      zeigt einzelnes rechteckiges Farbsymbol an
   ------------------------------------------------------- */
void symzif_print(uint8_t x, uint8_t y, uint16_t col)
{
  fillrect(x, y, x+9, y+9, rgbfromega(0));
  rectangle(x, y, x+9, y+9, rgbfromega(7));
  fillrect(x, y, x+7, y+7, col);
}

/* -------------------------------------------------------
                          symzahl_print

     zeigt eine Zahl als Farbe an.

     Uebergabe:
       pos  :  Ausgabeposition (0..8)
       num  :  anzuzeigende Zahl (3 oder 4 stellig)
       *sym :  Zeiger auf das Symbolarray
   ------------------------------------------------------- */
void symzahl_print(uint8_t pos, uint16_t num, uint8_t *sym, uint8_t *sel)
{
  uint8_t first;
  uint8_t xo, yo;
  uint16_t d, n;
  uint8_t ind;

  first= 1;
  xo= outpos_nums[pos*2];
  yo= outpos_nums[(pos*2)+1];
  if ( (pos == 2) || (pos == 5) || (pos == 8))
    d= 1000;                                    // 4-stellige Anzeie
  else
    d= 100;                                     // 3-stellige Anzeige
  while(d > 0)
  {
    n= num / d;                                 // Ziffer aus der Zahl extrahieren
    if (first && (!n) && (d > 1))               // fuehrende 0
    {
      xo += 10;                                 // nicht anzeigen
    }
    else
    {
      // Index suchen, an der die Ziffer in der Permutation steht.
      // Diese Position ist es dann, an der die Ziffer durch eine
      // Farbe ersetzt wird
      ind= 0;
      while ((n != sym[ind]) && (ind < 10)) ind++;

      // soll Farbe oder Ziffer angezeigt werden (Bit 7 = Anzeigeflag)
      if (!(sel[ind] & 0x80))
      {
        // Farbsymbol anzeigen
        symzif_print(xo, yo, cols565[ind]);
      }
      else
      {
        // Ziffer anzeigen
        zif_print(xo, yo, sel[ind] & 0x0f);
      }

      xo+= 10;
      first= 0;
    }
    num= num - (n * d);
    d /= 10;
  }
}

/* -------------------------------------------------------
                         operands_show

     zeigt die Symbole der Rechenoperationen (+ - = ) auf
     dem Display an
   ------------------------------------------------------- */
void operands_show(uint8_t *ops)
{
  uint8_t  xo, yo, i;

  textcolor= 0;
  for (i= 0; i< 36; i+= 3)
  {
    xo= ops[i];
    yo= ops[i+1];
    putcharxy(xo, yo, ops[i+2]);
  }
}

/* -------------------------------------------------------
                         raetsel_show

     zeigt das Raetsel an. Sind Ziffern den Farben schon
     zugeordnet, werden die Ziffern angezeigt
   ------------------------------------------------------- */
void raetsel_show(void)
{
  uint8_t i;

  for (i= 0; i< 9; i++)
    symzahl_print(i, anum[i], &colzif[0], &usersel[0]);
  operands_show(&outpos_ops[0]);
}

/* -------------------------------------------------------
                      sinterface_show

     zeigt das Auswahlinterface an
   ------------------------------------------------------- */
void sinterface_show(void)
{
  uint8_t        x;
  static uint8_t menusel = 0;

  if ((rectpos != 10) && menusel)
    button_draw(sym_ofsx+33, sym_ofsy+98, "Menu", fnt8x8, 0);

  // Zuordnungziffern anzeigen
  for (x= 0; x< 10; x++)
  {
    if ((usersel[x] & 0x80)== 0x80)
    {
      zif_print((x*12) + sym_ofsx - 1, sym_ofsy + 66, usersel[x] & 0x0f);
    }
    else
    {
      zif_print((x*12)+ sym_ofsx -1, sym_ofsy + 66, ' ' - '0');
      zif_print((x*12)+ sym_ofsx -1, sym_ofsy + 64, '.' - '0');       // weil das "." Zeichen weiter oben angezeigt wird
    }
  }
  if (rectpos < 10)                                                   // wenn Zuordnung selektiert ist
  {
    rectangle((rectpos*12) + sym_ofsx - 2, sym_ofsy + 64,             // Auswahlquadrat zeichnen
              (rectpos*12) + sym_ofsx + 9 ,sym_ofsy + 78, rectcol);
  }
  else
  {
    // Menu wurde selektiert
    menusel= 1;
    button_draw(sym_ofsx+33, sym_ofsy+98, "Menu", fnt8x8, 1);
  }
}

/* -------------------------------------------------------
                      colfeld_show

     zeigt die Farbkloetze der Auswahl sowie die
     Raetselnummer an
   ------------------------------------------------------- */
void colfeld_show(uint16_t nr)
{
  uint8_t x;

  for (x= 0; x< 10; x++)
    symzif_print((x*12) + sym_ofsx - 1, sym_ofsy+84, cols565[x]);     // Farbauswahl: farbige Rechtecke unten zeichnen

  textcolor= rgbfromega(0);
}

/* -------------------------------------------------------
                      usersel_input

     nimmt die Spieleeingabe waehrend der Zuordnung vor
     und gibt eine 1 zurueck, wenn der Menupunkt ausge-
     waehlt wurde.
     Ziffern werden den Farben zugeordnet. Wird eine
     andere Farbe angewaehlt, wird diese mit einem
     Rechteck umrandet.

     Rueckgabe:
       1 : wenn Menuwahl aktiviert wurde
   ------------------------------------------------------- */
uint8_t usersel_input(void)
{
  uint8_t b;
  uint8_t entry_free, ch;

  ch= key_get();
  switch (ch)
  {
    case (ENTER) :
    {
      if (rectpos == 10)
      {
        return 1;                         // Menu wurde ausgewaehlt
      }
      break;
    }

    case (UP) :
    {
      if (rectpos != 10)
      {
        do
        {
          b= usersel[rectpos];            // Ziffer lesen (incl. Flags)
          if (b == 0x89)                  // 0x80 | 0x09 => Flag + Ziffer 9
          {
            b= 0x40;                      // Ziffernzuordnungsflag loeschen
            usersel[rectpos]= b;
          }
          else
          {
            if (b== 0x40)
            {
              b= 0x80;                    // Zuordnung 0 setzen
              usersel[rectpos]= b;
            }
            else
              usersel[rectpos]= ++b;
          }
          entry_free= usersel_free(rectpos, b, &usersel[0]);
        // wiederhole solange, bis eine freie Zuordnung gewaehlt ist
        } while (!entry_free);
      }
      break;
    }

    case (DOWN)  :
    {
      if (rectpos != 10)                  // wenn Menu selektiert ist Downtaste sperren
      {
        do
        {
          b= usersel[rectpos];            // Ziffer lesen (incl. Flags)
          if (b == 0x80)                  // Flag + Ziffer 0
          {
            b= 0x40;                      // Ziffernzuordnungsflag loeschen
            usersel[rectpos]= b;
          }
          else
          {
            if (b== 0x40)
            {
              b= 0x89;                    // Zuordnung 9 setzen
              usersel[rectpos]= b;
            }
            else
              usersel[rectpos]= --b;
          }
          entry_free= usersel_free(rectpos, b, &usersel[0]);
        // wiederhole solange, bis eine freie Zuordnung gewaehlt ist
        } while (!entry_free);
      }
      break;
    }
    case (RIGHT) :
    {
      if (rectpos < 10)
      {
        // Auswahlquadrat mit Hintergrundfarbe loeschen
        rectangle((rectpos*12) + sym_ofsx - 2, sym_ofsy + 64,
                  (rectpos*12) + sym_ofsx + 9 ,sym_ofsy + 78, bkscrcolor);
      }
      rectpos++;
      rectpos = rectpos % 11;
      break;
    }
    case (LEFT)  :
    {
      if (rectpos < 10)
      {
        // Auswahlquadrat mit Hintergrundfarbe loeschen
        rectangle((rectpos*12) + sym_ofsx - 2, sym_ofsy + 64,
                  (rectpos*12) + sym_ofsx + 9 ,sym_ofsy + 78, bkscrcolor);
      }
      if (rectpos > 0) rectpos--; else rectpos= 10;
    }
    default      : break;
  }
  return 0;
}

/* -------------------------------------------------------
                        keyb_getint


     liest mittels 4 Tasten einen Integerwert ein .
     Links-rechtstasten waehlen die Ziffer aus, die ein-
     gestellt wird. Die selektierte Ziffer wird mitteols
     eines Rechtecks umrandet. Hoch-runtertaste veraendert
     den Wert der selektierten Ziffer.

     Uebergabe:
       x,y    : Koordinate, ab der ein Integerwert einge-
                geben wird.
       aktw   : Zahlenwert, der beim Eintritt in diese
                Funktion als Ausgangswert vorgegeben wird.
       digits : wieviele Ziffern sollen eingestellt werden
                koennen.

     Rueckgabe : eingelesener Integerwert

     Usage:
      i = keyb_getint(50,64, 0, 3);

     ------------------------------------------------------- */
uint16_t keyb_getint(uint16_t x, uint16_t y, uint16_t aktw, uint8_t digits)
{
  int16_t  rp;
  uint16_t value;
  uint8_t  vziff[5];
  int16_t  w,i;
  uint8_t  ch;


  value= aktw;
  w= 10000;
  for (i= 4; i> -1; i--)
  {
    vziff[i]= value / w;
    value -= vziff[i] * w;
    w= w / 10;
  }

  for (rp= 0; rp< digits+1; rp++)
  {
    zif_print( ((digits-rp)*12)+x,y, vziff[rp] );
  }

  rp= digits;
  rectangle( ((digits-rp)*12) + x - 2, y-2, ((digits-rp)*12) + x + 11 ,y+11, rectcol);

  while(1)
  {
    ch= key_get();
    switch (ch)
    {
      case (ENTER) :
      {
        value = (vziff[4]*10000) + (vziff[3]*1000) + (vziff[2]*100) + (vziff[1]*10) + vziff[0];
        return value;
        break;
      }
      case (UP) :
      {
        if (vziff[rp]< 9) { vziff[rp]++; } else { vziff[rp]= 0; }
        zif_print( ((digits-rp)*12)+x,y, vziff[rp] );
        break;
      }
      case (DOWN) :
      {
        if (vziff[rp]> 0) { vziff[rp]--; } else { vziff[rp]= 9; }
        zif_print( ((digits-rp)*12)+x,y, vziff[rp] );
        break;
      }
      case (RIGHT) :
      {
        rectangle( ((digits-rp)*12) + x - 2, y-2, ((digits-rp)*12) + x + 11 ,y+11, bkscrcolor);
        if (rp> 0) { rp--; } else { rp= digits; }
        rectangle( ((digits-rp)*12) + x - 2, y-2, ((digits-rp)*12) + x + 11 ,y+11, rectcol);
        break;
      }
      case (LEFT) :
      {
        rectangle( ((digits-rp)*12) + x - 2, y-2, ((digits-rp)*12) + x + 11 ,y+11, bkscrcolor);
        if (rp< digits) { rp++; } else { rp= 0; }
        rectangle( ((digits-rp)*12) + x - 2, y-2, ((digits-rp)*12) + x + 11 ,y+11, rectcol);
        break;
      }
      default: break;
    }
  }
}

/* -------------------------------------------------------
                        menu_show

     zeigt Schwierigkeitsstufen als Text an

     Uebergabe:
        xofs, yofs : Koordinaten, an der der Screen
                     gezeichnet wird
        select     : Menupunkt der als selektiert aus-
                     gewaehlt ist

   ------------------------------------------------------- */
void menu_show(uint16_t xofs, uint16_t yofs, uint8_t select)
{
  uint16_t col_unsel, col_sel;

  col_unsel= rgbfromvalue(0xa0, 0xa0, 0xa0);
  col_sel= rgbfromvalue(0xff, 0xfb, 0x2a);
  setfont(1);

  if (select == 0) textcolor= col_sel; else textcolor= col_unsel;
  outtextxy(xofs, yofs, 0,"Neu");
  if (select == 1) textcolor= col_sel; else textcolor= col_unsel;
  outtextxy(xofs+2, yofs+18, 0,"Load");
  if (select == 2) textcolor= col_sel; else textcolor= col_unsel;
  outtextxy(xofs+2, yofs+36, 0,"Save");
  if (select == 3) textcolor= col_sel; else textcolor= col_unsel;
  outtextxy(xofs+2, yofs+54, 0,"Auswahl");
  if (select == 4) textcolor= col_sel; else textcolor= col_unsel;
  outtextxy(xofs+2, yofs+72, 0,"Loesung");
  if (select == 5) textcolor= col_sel; else textcolor= col_unsel;
  outtextxy(xofs+2, yofs+90, 0,"Zurueck");
}


/* -------------------------------------------------------
                       get_menu

     waehlt aus einem Menupunkt aus

     Uebergabe:
        xofs, yofs : Koordinaten, an der der Screen
                     gezeichnet wird

     Rueckgabe: gewaehlter Menupunkt
------------------------------------------------------- */
uint8_t menu_get(uint16_t xofs, uint16_t yofs)
{
  uint8_t ch, sel;

  bkcolor= rgbfromvalue(0x00, 0x00, 0x50);
  clrscr();
  spiro_generate(80, 200, 70, 80, 140,512, rgbfromvalue(0x00,0x00, 0x90));
  bkcolor= bkscrcolor;

  sel= 0;
  do
  {
    menu_show(xofs, yofs, sel);
    ch= key_get();
    if (ch== 1)                         // Taste nach oben
    {
      if (sel> 0) sel--; else sel= 5;
    }
    if (ch== 4)                         // Taste nach unten
    {
      if (sel< 5) sel++; else sel= 0;
    }
  } while(ch != 5);
  setfont(0);
  return sel+1;
}

#define  gaddr  0x0020    // Adresse im EEProm, ab dem der Spielzustand
                          // gespeichert wird

/* ----------------------------------------------------
                      game_save

  speichert das aktuelle Raetsel auf dem EEProm,
  folgende Arrays werden geladen:


  uint16_t anum[9];    // nimmt die 9 Integerzahlen der
                       // Rechenoperation auf
                       // und somit das Raetsel

  uint8_t  colzif[10]; // nimmt die Farbzuweisung
                       // einer Farbe zu einer Ziffer auf

  uint8_t usersel[10]; // Nimmt die Ziffern auf, die
                       // der User den Farben zuordnet

  ------------------------------------------------------- */
void game_save(void)
{
  uint16_t   i;

  flash_init();
  for (i= 0; i< 9; i++)
  {
    eep_write16((gaddr+(i * 2)), anum[i]);
    delay(4);
  }
  eep_write16((gaddr+18), 0x00);    // Kennung, dass ein Raetsel gespeichert ist
  delay(4);
  for (i= 0; i< 10; i++)
  {
    eep_write16((gaddr+20+i), colzif[i]);
    delay(4);
    eep_write16((gaddr+30+i), usersel[i]);
    delay(4);
  }
  eep_write16((gaddr+42), zstw);
  delay(4);
}

void game_load(void)
{
  uint16_t   i;

  if (eep_read16((gaddr+18)) == 0)
  {
    delay(4);
    for (i= 0; i< 9; i++)
    {
      anum[i]= eep_read16((gaddr+(i * 2)));
      delay(4);
    }
    for (i= 0; i< 10; i++)
    {
      colzif[i]= eep_read16((gaddr+20+i));
      delay(4);
      usersel[i]= eep_read16((gaddr+30+i));
      delay(4);
    }
    zstw= eep_read16((gaddr+42));
    delay(4);
    newraetsel= 0;
  }
}

/* ----------------------------------------------------
                       raetsel_gen

  generiert ein neues Raetsel
------------------------------------------------------- */
void raetsel_gen(void)
{
  uint8_t i;

  start_state= 1;
  lfsr= 1;
  first = 0;

  zstw= zstw % 10000;

  rectpos= 0;
  for (i= 0; i< 10; i++) usersel[i]= 0x40;    // alle Farbzuordnungen loeschen

  rndm_permut10(&colzif[0], zstw);            // Farbenzuweisung "mischen"
  do
  {
    generate_termnums(&anum[0], zstw);        // und ein neues Raetsel erstellen, zstw ist Raetselnummer
    ziffcheck= ziffanz_check();               // feststellen ob alle Ziffern 0..9 im Raetsel vorhanden sind
  } while (ziffcheck != 0x03ff);              // solange Raetsel generieren, bis alle Ziffern vorhanden
}

/* ----------------------------------------------------
                     raetsel_start

  wertet die Auswahl des Menus aus und setzt ent-
  sprechend die Parameter fuer das Spiel
------------------------------------------------------- */
void raetsel_start(uint8_t value)
{
  switch(value)
  {

    // neues zufaelliges Raetsel
    case 1 :
    {
      zstw++;
      if (zstw== 10000) { zstw= 1; }
      raetsel_gen();
      newraetsel= 0;
      break;
    }

    // Raetsel laden
    case 2 :
    {
      if (eep_read16((gaddr+18)) == 0)
      {
        game_load();
      }
      else              // wenn kein Raetsel gespeichert ist, ein neues generieren
      {
        zstw++;
        if (zstw== 10000) { zstw= 1; }
        raetsel_gen();
        newraetsel= 0;
      }
      break;
    }

    // Raetsel speichern
    case 3 :
    {
      bkcolor= rgbfromvalue(0x00, 0x00, 0x50);
      clrscr();
      spiro_generate(80, 200, 70, 80, 140,512, rgbfromvalue(0x00,0x00, 0x90));
      bkcolor= bkscrcolor;

      game_save();
      textcolor= rgbfromega(15);
      outtextxy(36,63,0, "Raetsel");
      outtextxy(20,73,0, "gespeichert");
      delay(1500);
      break;
    }

    // Eingabe einer Raetselnummer
    case 4 :
    {
      clrscr();

      spiro_generate(80, 200, 70, 80, 140,512, rgbfromvalue(0x00,0x00, 0x90));
      rectangle(sym_ofsx-6,0, sym_ofsx+121, 127, rgbfromega(15));
      setfont(1);
      textcolor= rgbfromega(14);
      outtextxy(sym_ofsx,12,0,"Eingabe");
      outtextxy(sym_ofsx,30,0,"Raetselnr.");
      setfont(0);
      zstw= keyb_getint(sym_ofsx+38,63,0, 3);
      if (zstw== 0) { zstw++; }
      raetsel_gen();
      newraetsel= 0;
      break;
    }

    // Loesung anzeigen
    case 5 :
    {
      // Term in rechnerischer Darstellung
      clrscr();
      raetsel_show();
      textcolor= rgbfromega(0);
      gotoxy(0,7);

      textcolor= rgbfromvalue(0x00, 0x00, 0x80);

      #if (_yres> 128)
        printf("\n\r   %d+%d = %d", anum[0], anum[1], anum[2]);
        printf("\n\r    -   +      -");
        printf("\n\r   %d-%d = %d", anum[3], anum[4], anum[5]);
        printf("\n\r   --------------");
        printf("\n\r   %d+%d = %d", anum[6], anum[7], anum[8]);
        printf("\n\n\r");
        textcolor= rectcol;
        printf("   Taste rechts...");
      #else
        printf("\n\r %d+%d = %d", anum[0], anum[1], anum[2]);
        printf("\n\r  -   +      -");
        printf("\n\r %d-%d = %d", anum[3], anum[4], anum[5]);
        printf("\n\r --------------");
        printf("\n\r %d+%d = %d", anum[6], anum[7], anum[8]);
        printf("\n\n\r");
        textcolor= rectcol;
        printf(" Taste rechts...");
      #endif
      textcolor= 0;
      rectangle(sym_ofsx-6,0,sym_ofsx+121,127, rgbfromega(15));
      while( !(is_keyright()));
      delay(50);
      while( is_keyright());
      delay(50);

//      if (geloest) { newraetsel= 1; }

    }
    default : break;
  }
}

/* ---------------------------------------------------------------------------
                                    M A I N
   --------------------------------------------------------------------------- */
int main(void)
{
//  int      i;
  uint8_t  menuwahl, geloest;
  uint8_t  mvalue;

  char     strbuffer[32];

  sys_init();

  // Display initialisieren und Grundeinstellungen vornehmen
  lcd_init();

  clrscr();
  textsize= 0;
  setfont(0);
  lcd_orientation(1);

  key_init();

  bkscrcolor= rgbfromvalue(0xa4, 0x84, 0x60);
  rectcol= rgbfromvalue(0xff, 0xd0, 0xd0);

  bkcolor= rgbfromvalue(0x00, 0x00, 0x84);
  clrscr();

  #if (_yres > 128)
    pcx256_show(16,0, &codedcolorscreen[0], &codedcolorpal[0]);
  #else
    pcx256_show(0,0, &codedcolorscreen[0], &codedcolorpal[0]);
  #endif
  bkcolor= bkscrcolor;

  textcolor= rgbfromega(7);
  outtextxy(10,113, 0, "Press up-key");
  textcolor= rgbfromega(0);
  zstw= 0;

  do
  {
    zstw++;                            // zstw wird solange hochgezaehlt
    zstw= zstw % 10000;
  } while( !(is_keyup()));              // bis eine Taste gedruckt ist (somit zufaellig)
  delay(50);
  while( is_keyup());
  delay(50);
  newraetsel= 1;

  bkcolor= rgbfromvalue(0x00, 0x00, 0x50);
  clrscr();
  spiro_generate(80, 200, 70, 80, 140,512, rgbfromvalue(0x00,0x00, 0x90));

  do
  {
    mvalue= menu_get(22,11);
    if ((mvalue== 1) || (mvalue== 2) || (mvalue== 4))
    {
      raetsel_start(mvalue);
    }
  } while (!((mvalue == 1) || (mvalue == 2) || (mvalue == 4)));

  while(1)
  {
    clrscr();

    rectangle(sym_ofsx-6,0, sym_ofsx+121, 127, rgbfromega(15));

    if (newraetsel)
    {
      zstw++;
      raetsel_gen();
      newraetsel= 0;
    }

    button_draw(sym_ofsx+33, sym_ofsy+98, "Menu", fnt8x8, 0);

    do
    {
      raetsel_show();                         // Zeigt Raetsel mit gemachten Ersetzungen an
      colfeld_show(zstw);
      sinterface_show();

      textcolor= rgbfromvalue(0x00, 0x00, 0x80);

      my_sprintf(strbuffer,"Raetsel: %d",zstw);
      outtextxy(sym_ofsx+4,60,0,strbuffer);
      textcolor= 0;

      menuwahl= usersel_input();
      geloest= zif_cmp();
    } while(!menuwahl && !geloest);

    if (geloest)
    {
      clrscr();

      bkcolor= rgbfromvalue(0x00, 0x00, 0x50);
      clrscr();
      spiro_generate(80, 200, 70, 80, 140,512, rgbfromvalue(0x00,0x00, 0x90));

      setfont(1);
      textcolor= rgbfromega(14);
      outtextxy(sym_ofsx+22,40,0,"Raetsel");
      outtextxy(sym_ofsx+22,60,0,"geloest");
      setfont(0);
      newraetsel= 1;
      delay(1500);
    }

    do
    {
      mvalue= menu_get(22,11);
      raetsel_start(mvalue);
    } while ((mvalue == 5) || (mvalue == 3));   // bei Auswahl, Save und Loesung im Menu verbleiben

  }
}

/*
  // Loesung fuer Raetselnr. 1000, nur zu Debugzwecken

  zstw= 1000;

  usersel[0] = 0x03 | 0x80;
  usersel[1] = 0x02 | 0x80;
  usersel[2] = 0x00 | 0x80;
  usersel[3] = 0x08 | 0x80;
  usersel[4] = 0x07 | 0x80;
  usersel[5] = 0x01 | 0x80;
  usersel[6] = 0x09 | 0x80;
  usersel[7] = 0x05 | 0x80;
  usersel[8] = 0x04 | 0x80;
  usersel[9] = 0x06 | 0x80;
*/
